package pl.poznan.put.cs.idss.jrs.test;

import java.io.IOException;
import java.util.ArrayList;
import pl.poznan.put.cs.idss.jrs.approximations.MonotonicUnionContainer;
import pl.poznan.put.cs.idss.jrs.approximations.UnionContainer;
import pl.poznan.put.cs.idss.jrs.classifiers.AllRulesVCDRSAClassificationMethod;
import pl.poznan.put.cs.idss.jrs.classifiers.ClassificationResultsContainer;
import pl.poznan.put.cs.idss.jrs.classifiers.ClassificationResultsValidationContainer;
import pl.poznan.put.cs.idss.jrs.classifiers.Classifier;
import pl.poznan.put.cs.idss.jrs.classifiers.RulesVCDRSAClassificationMethod;
import pl.poznan.put.cs.idss.jrs.core.InvalidValueException;
import pl.poznan.put.cs.idss.jrs.core.mem.MemoryContainer;
import pl.poznan.put.cs.idss.jrs.rules.MonotonicVCDomLem;
import pl.poznan.put.cs.idss.jrs.rules.Rule;
import pl.poznan.put.cs.idss.jrs.rules.RulesContainer;
import pl.poznan.put.cs.idss.jrs.rules.SimpleConditionValidator;
import pl.poznan.put.cs.idss.jrs.rules.UnionDecisionsPredictor;
import pl.poznan.put.cs.idss.jrs.rules.VCDomLem;
import pl.poznan.put.cs.idss.jrs.utilities.ISFLoader;

/**
 * Main class
 * 
 * @author Marcin Szeląg
 */
public class Main {

	/**
	 * Application entry point
	 * 
	 * @param args command line arguments
	 */
	public static void main(String[] args) {
		String dataFile = "data/denbosch.isf";
		MemoryContainer decisionTable;
		UnionContainer unionContainer;
		double consistencyThreshold = 0.0; //threshold for a cost-type consistency measure $\epsilon$; value 0.0 corresponds to classical DRSA
		
		//load decision table from file
		decisionTable = ISFLoader.loadISFIntoMemoryContainer(dataFile);
		if (decisionTable == null) {
			System.out.println("Could not read data from " + dataFile + " file.");
			return;
		} else {
			System.out.println("Loaded "+dataFile);
		}
		
		//create union container for learning data table
		try {
			unionContainer = new MonotonicUnionContainer(decisionTable); //guess decision criterion number
		}
		catch (InvalidValueException exception) {
			System.out.println(exception.getMessage()); //!= 1 active decision criterion
			return;
		}
		
		//write approximations to file
		String apxFile = dataFile.replaceFirst(".isf", ".apx"); //prepare name of the file
		try {
			unionContainer.writeApproximations(apxFile, consistencyThreshold, true, "This is a dummy comment");
			System.out.println("Approximations written to " + apxFile + " file.");
		}
		catch (IOException exception) {
			System.out.println("Could not write approximations to " + apxFile + "file.");
			System.out.println(exception.getMessage());
		}
		
		//create empty rules container
		RulesContainer rulesContainer = new RulesContainer(decisionTable, consistencyThreshold);
		//create rule generator
		VCDomLem vcDomLem = new MonotonicVCDomLem();
		
		ArrayList<Rule> atLeastRules = vcDomLem.generateRules(
				unionContainer.getUpwardUnions(),
				consistencyThreshold,
				new UnionDecisionsPredictor(),
				Rule.CERTAIN,
				Rule.AT_LEAST,
				new SimpleConditionValidator(),
				decisionTable,
				VCDomLem.MIX_CONDITIONS_FROM_DIFFERENT_OBJECTS,
				VCDomLem.COVER_NONE_OF_NEGATIVE_EXAMPLES);
		
		ArrayList<Rule> atMostRules = vcDomLem.generateRules(
				unionContainer.getDownwardUnions(),
				consistencyThreshold,
				new UnionDecisionsPredictor(),
				Rule.CERTAIN,
				Rule.AT_MOST,
				new SimpleConditionValidator(),
				decisionTable,
				VCDomLem.MIX_CONDITIONS_FROM_DIFFERENT_OBJECTS,
				VCDomLem.COVER_NONE_OF_NEGATIVE_EXAMPLES);
		
		//store induced rules in the rules container 
		for (int i = 0; i < atLeastRules.size(); i++) {
			rulesContainer.storeRule(atLeastRules.get(i));
		}
		for (int i = 0; i < atMostRules.size(); i++) {
			rulesContainer.storeRule(atMostRules.get(i));
		}
		
		//write rules to file
		String rulesFile = dataFile.replaceFirst(".isf", ".rules"); //prepare name of the file
		try {
			rulesContainer.writeRules(rulesFile, true, true);
			System.out.println("Induced rules written to " + rulesFile + " file.");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//load test information table from file
		MemoryContainer testDecisionTable = ISFLoader.loadISFIntoMemoryContainer(dataFile);
		if (testDecisionTable == null) {
			System.out.println("Could not read data from " + dataFile + " file.");
			return;
		} else {
			System.out.println("Loaded " + dataFile);
		}
		
		//create objects responsible for classification of examples from the test information table
		AllRulesVCDRSAClassificationMethod vcdrsaMethodOnlyCertain = 
				new AllRulesVCDRSAClassificationMethod(testDecisionTable, rulesContainer);
		vcdrsaMethodOnlyCertain.setRuleType(RulesVCDRSAClassificationMethod.ONLY_CERTAIN);
		
		Classifier classifierOnlyCertain = 
				new Classifier(vcdrsaMethodOnlyCertain, rulesContainer.getNumberOfUsedActiveDecisionAttribute());
		
		ClassificationResultsContainer classificationResultsContainerOnlyCertain =
				new ClassificationResultsValidationContainer(
						classifierOnlyCertain, decisionTable, testDecisionTable
				);
		
		//classify all examples from the test information table
		/*Vector<ClassificationResult> classificationResultsOnlyCertain = */
		classificationResultsContainerOnlyCertain.classifyAllObjects();
		
		String classificationFile = dataFile.replaceFirst(".isf", ".cls"); //prepare name of the file
		String classificationMatrixFile = dataFile.replaceFirst(".isf", ".mtx"); //prepare name of the file
		
		//write classification results and misclassification matrix
		try {
			((ClassificationResultsValidationContainer)classificationResultsContainerOnlyCertain)
					.writeClassificationResultsRAW(classificationFile);
			System.out.println("Classification results written to " + classificationFile + " file.");
			
			((ClassificationResultsValidationContainer)classificationResultsContainerOnlyCertain)
					.writeMisclassificationMatrix(classificationMatrixFile);
			System.out.println("Misclassification matrix written to " + classificationMatrixFile + " file.");
		} catch (IOException exception){
			System.out.println("Could not write classification results to " + classificationFile + "file.");
			System.out.println(exception.getMessage());
		}
		
	}

}
